package Pattern;


=head1 NAME

package Pattern;

=cut


require Exporter;
@ISA = Exporter;
@EXPORT = qw(ParasePattern PatternLIE PatternPEI);


=head2 ParasePattern

=head2 ParasePattern_SYNOPSIS

=over 4
use Pattern;

%HoP = (
	);

ParasePattern(C<Pattern_Str>,C<Hash_of_Array>);

  ParasePattern("Search+Phrase-NotThis",\%HoP);
  ParasePattern("th(is|at)+th[eo]se",\%HoP);
  ParasePattern('\bc\b-\bc\+\++pascal',\%HoP);

=back

The parameters are:

=over 4

=item Pattern_Str 

Pattern string to be Parased. see L<ParasePattern_SYNOPSIS>.

=item Hash_of_Array

On return:

%HoP{incl}
  contains the array for included pattern.

%HoP{excl}
  contains the array for excluded pattern.

=back

=cut

sub ParasePattern {
	my $PatternStr = shift (@_);
	$HoP	 = shift (@_);

	my $NdxInc	= 0;
	my $NdxExcl = 0;
	
	$PatternStr = '+' . $PatternStr 
		if $PatternStr !~ /^[-+]/ ; # Actually unnecessary for the	
					    # following match pattern
	#print STDERR "$PatternStr\n";
	while( $PatternStr =~ /([-+])([^-+]+)/ ){
		#print STDERR "$1:'$2'\n";
		my $pattern_type = $1;
		my $pattern_part = $2;
		$PatternStr=$';
		
		while (substr($pattern_part,-1,1) eq '\\'){
			# concat parts after the next +/- 
			chop($pattern_part);
			
			$PatternStr =~ /([-+])([^-+]*)/;
			$pattern_part .= $1 . $2;
			#print STDERR "$pattern_part\n";
			$PatternStr=$';
			#print STDERR "$PatternStr\n";
			}
		
		if ($pattern_type eq '+' ){
			$$HoP{incl}[$NdxInc++]=$pattern_part;
			}
		elsif ($pattern_type eq '-' ){
			$$HoP{excl}[$NdxExcl++]=$pattern_part;
			}
		else {	# wrong !
			die "Internal error when Parasing Pattern\n";
			}
		};
	
	1;
	if ( !defined($$HoP{incl}[0]) ) {
		print STDERR "Fatal error: no pattern found\n";
		exit(1);
		}
		
	#print STDERR "--I:@{$$HoP{incl}}\n--E:@{$$HoP{excl}}\n";
	}

=head2 PatternLIE

Pattern match, Line oriented, Include than Exclude

PatternLIE(\C<Hash_of_Array>,\@lines,\&my_callback);
 PatternLIE(\%HoP,\@lines,\&my_callback);

  sub my_callback {
	  my $org_line=shift;
	  my $l_no=shift;
	  $org_line=$$org_line;
	  ...
	  }

=cut

sub PatternLIE {
	my $HoP     = shift;
	my $arr_ref = shift;
	my $callback = shift;
	
	
	my $l_no=0;
	LINE: 
	foreach (@$arr_ref){
		my $found = 0;		# Actually is the times of found.

		++$l_no;
		# == First, include.
		foreach my $patt (@{$$HoP{incl}}){
			next unless /$patt/i;
			$found++;
			}
		next LINE unless $found;
		#print STDERR "PatternIE found: $_\n";

		# == Then, exclude.
		foreach my $patt (@{$$HoP{excl}}){
			#print STDERR "PatternIE excl: $patt\n";
			next LINE if /$patt/i;
			}
		
		&$callback(\$_, $l_no);
		}
	}
	

=head2 PatternPEI

Pattern match, Paragraph oriented, Exclude than Include

PatternPEI(\C<Hash_of_Array>,\@lines,\&my_callback);
  PatternPEI(\%HoP,\@lines,\&my_callback);

  sub my_callback {
	  my $org_line=shift;
	  my $found_pattern=shift;
	  $org_line=$$org_line;
	  ...
	  }

=cut

sub PatternPEI {
	my $HoP     = shift;
	my $arr_ref = shift;
	my $callback = shift;
	my $found = 0;		# Actually is the times of found.
	
	LINE: 
	foreach $line (@$arr_ref){

		# == First, exclude.
		# boom out if any excludes has been found in the *whole paragraph*
		foreach my $patt (@{$$HoP{excl}}){
			if ( $line =~ /$patt/i ){
				#warn "$patt:$line\n";
				return 0;}
			}
		
		# == Then, include.
		foreach my $patt (@{$$HoP{incl}}){
			next unless $line =~ /$patt/i;
			$found++;
			&$callback(\$line,$patt) if $found;
			}
		}
	return $found;
	}
	
1;

=head1 AUTHOR

Sun Tong E<lt>F<suntong@goecities.com>E<gt>

=head1 COPYRIGHT

Copyright (c) 1997 Sun Tong. All rights reserved.

This program is freeware, but requires you to register, just send me an email 
:-). I want to know where my program goes.

Register syntax:

 To:	  suntong@geocities.com
 Subject: Pattern <Your Country> <Your state>
 Body:	  <Maybe blank, Any comments, bug reports, etc, are
	  welcome>


=head1 TODO

=cut

